#!/usr/bin/python3

import csv, sys
import itertools
from pprint import pprint


candidates = [
    "Nathalie Arthaud",
    "François Asselineau",
    "Jacques Cheminade",
    "Nicolas Dupont-Aignan",
    "François Fillon",
    "Benoît Hamon",
    "Jean Lassalle",
    "Marine Le Pen",
    "Emmanuel Macron",
    "Jean-Luc Mélenchon",
    "Philippe Poutou"
]

c_index = {
    "Nathalie Arthaud": 0,
    "François Asselineau": 1,
    "Jacques Cheminade": 2,
    "Nicolas Dupont-Aignan": 3,
    "François Fillon": 4,
    "Benoît Hamon": 5,
    "Jean Lassalle": 6,
    "Marine Le Pen": 7,
    "Emmanuel Macron": 8,
    "Jean-Luc Mélenchon": 9,
    "Philippe Poutou": 10
}

cc_order = [
    "Nicolas Dupont-Aignan",
    "Marine Le Pen",
    "Emmanuel Macron",
    "Benoît Hamon",
    "Nathalie Arthaud",
    "Philippe Poutou",
    "Jacques Cheminade",
    "Jean Lassalle",
    "Jean-Luc Mélenchon",
    "François Asselineau",
    "François Fillon"
]

perm = [candidates.index(c) for c in cc_order]

nb_candidates = len(candidates)

official = [
    {
        "Jean-Luc Mélenchon": 362,
        "Emmanuel Macron": 261,
        "Benoît Hamon": 134,
        "François Fillon": 88,
        "Marine Le Pen": 45,
        "Nicolas Dupont-Aignan": 12,
        "Nathalie Arthaud": 5,
        "Philippe Poutou": 5,
        "Jean Lassalle": 3,
        "François Asselineau": 3,
        "Jacques Cheminade": 0
    },
    {
        "Jean-Luc Mélenchon": 280,
        "Emmanuel Macron": 176,
        "Benoît Hamon": 73,
        "François Fillon": 171,
        "Marine Le Pen": 54,
        "Nicolas Dupont-Aignan": 9,
        "Nathalie Arthaud": 3,
        "Philippe Poutou": 5,
        "Jean Lassalle": 4,
        "François Asselineau": 5,
        "Jacques Cheminade": 1
    },
    {
        "Jean-Luc Mélenchon": 143,
        "Emmanuel Macron": 247,
        "Benoît Hamon": 68,
        "François Fillon": 164,
        "Marine Le Pen": 52,
        "Nicolas Dupont-Aignan": 12,
        "Nathalie Arthaud": 3,
        "Philippe Poutou": 5,
        "Jean Lassalle": 5,
        "François Asselineau": 4,
        "Jacques Cheminade": 0
    }
]

official_all = {}
for cand in candidates:
    official_all[cand] = sum(vt[cand] for vt in official)
    
official_national = {
    "Nathalie Arthaud": 232428,
    "François Asselineau": 332588,
    "Jacques Cheminade": 65586,
    "Nicolas Dupont-Aignan": 1695000,
    "François Fillon": 7212995,
    "Benoît Hamon": 2291288,
    "Jean Lassalle": 435301,
    "Marine Le Pen": 7678491,
    "Emmanuel Macron": 8656346,
    "Jean-Luc Mélenchon": 7059951,
    "Philippe Poutou": 394505
}


vote_labels = {
    "NDA": "Nicolas Dupont-Aignan",
    "MLP": "Marine Le Pen",
    "EM": "Emmanuel Macron",
    "BH": "Benoît Hamon",
    "NA": "Nathalie Arthaud",
    "PP": "Philippe Poutou",
    "JC": "Jacques Cheminade",
    "JL": "Jean Lassalle",
    "JLM": "Jean-Luc Mélenchon",
    "FA": "François Asselineau",
    "FF": "François Fillon",
    "NSPP": None,
    "B": "Blanc"
}

#### CHARGEMENT DES DONNÉES ###########################

vt = [[], [], []]

for p in range(3):
    with open("../vt{}.tsv".format(p + 1), "r") as file:
        header = True
        nb_voters = 0
        for row in csv.reader(file, delimiter='\t'):
            if row[0][0] != '#':
                if not header:
                    vt[p].append({"approval": {}, "evaluation": {}})
                    for k in range(nb_candidates):
                        cand = candidates[perm[k]]
                        vt[p][-1]["approval"][cand] = int(row[k + 1])
                        vt[p][-1]["evaluation"][cand] = (
                            float(row[k + 12]) if row[k + 12] != 'None' else None)
                    vt[p][-1]["official"] = vote_labels[row[23]]
                    vt[p][-1]["age"] = row[24]
                    vt[p][-1]["gender"] = row[25]
                    vt[p][-1]["studies"] = row[26]
                    vt[p][-1]["prof"] = row[27]
                    vt[p][-1]["approval_appreciation"] = row[28]
                    vt[p][-1]["evaluation_appreciation"] = row[29]
                    vt[p][-1]["official_appreciation"] = row[30]
                    nb_voters += 1
            header = False
            
#### CALCUL DE LA CORRECTION ###########################

nb_voters = [len(vt[k]) for k in range(3)]
nb_voters_filtered = [sum(1 for v in vt[k] if v["official"] in candidates) for k in range(3)]
nb_voters_official = [sum(official[k][c] for c in official[k]) for k in range(3)]
nb_voters_official_national = sum(official_national[c] for c in official_national)

nb_voters_all = sum(nb_voters)
nb_voters_filtered_all = sum(nb_voters_filtered)
nb_voters_official_all =  sum(nb_voters_official)
nb_voters_official_national_all = nb_voters_official_national

for cand in candidates:
    print(cand, 100 * official_national[cand] / nb_voters_official_national_all)

print(sum(100 * official_national[cand] / nb_voters_official_national_all for cand in candidates))

official_order = [sorted(candidates,
                        key=lambda cand: official[k][cand], reverse=True) for k in range(3)]
official_order_all = sorted(candidates,
                            key=lambda cand: official_all[cand], reverse=True)
official_national_order = sorted(candidates,
                                 key=lambda cand: official_national[cand], reverse=True)

corrections = [{}, {}, {}]
corrections_all = {}
corrections_national = [{}, {}, {}]
corrections_national_all = {}

for k in vote_labels:
    if vote_labels[k] is not None:
        if vote_labels[k] in candidates:
            corrections_all[vote_labels[k]] = (
                (official_all[vote_labels[k]] * nb_voters_filtered_all)
                / (sum(sum(1 for v in vt[p] if v["official"] == vote_labels[k]) for p in range(3))
                   * nb_voters_official_all)
                if sum(sum(1 for v in vt[p] if v["official"] == vote_labels[k]) for p in range(3)) > 0 else 0)
            corrections_national_all[vote_labels[k]] = (
                (official_national[vote_labels[k]] * nb_voters_filtered_all)
                / (sum(sum(1 for v in vt[p] if v["official"] == vote_labels[k]) for p in range(3))
                   * nb_voters_official_national_all)
                if sum(sum(1 for v in vt[p] if v["official"] == vote_labels[k]) for p in range(3)) > 0 else 0)
            for p in range(3):
                corrections[p][vote_labels[k]] = (
                    (official[p][vote_labels[k]] * nb_voters_filtered[p])
                    / (sum(1 for v in vt[p]
                           if v["official"] == vote_labels[k]) * nb_voters_official[p])
                    if sum(1 for v in vt[p] if v["official"] == vote_labels[k]) > 0 else 0)
                corrections_national[p][vote_labels[k]] = (
                    (official_national[vote_labels[k]] * nb_voters_filtered[p])
                    / (sum(1 for v in vt[p]
                           if v["official"] == vote_labels[k]) * nb_voters_official_national)
                    if sum(1 for v in vt[p] if v["official"] == vote_labels[k]) > 0 else 0)

pprint(corrections_national_all)
                

#### CALCUL DES RANGS ###########################

approval_ranks = [{}, {}, {}]

for p in range(3):
    for cand in candidates:
        approval_ranks[p][cand] = {}
        approval_ranks[p][cand]["Score brut"] = sum(v["approval"][cand] for v in vt[p])
        approval_ranks[p][cand]["Score corrigé"] = sum(corrections[p][v["official"]] * v["approval"][cand] for v in vt[p] if v["official"] in candidates)
        approval_ranks[p][cand]["Score corrigé national"] = sum(corrections_national[p][v["official"]] * v["approval"][cand] for v in vt[p] if v["official"] in candidates)
        approval_ranks[p][cand]["Moyenne du score brut"] = approval_ranks[p][cand]["Score brut"] / nb_voters[p]
    raw_order = sorted(candidates,
                       key=lambda cand: approval_ranks[p][cand]["Score brut"], reverse=True)
    corrected_order = sorted(candidates,
                             key=lambda cand: approval_ranks[p][cand]["Score corrigé"], reverse=True)    
    corrected_order_national = sorted(candidates,
                                      key=lambda cand: approval_ranks[p][cand]["Score corrigé national"], reverse=True)
    for cand in candidates:
        approval_ranks[p][cand]["Rang brut"] = raw_order.index(cand) + 1
        approval_ranks[p][cand]["Rang corrigé"] = corrected_order.index(cand) + 1
        approval_ranks[p][cand]["Rang corrigé national"] = corrected_order_national.index(cand) + 1
        approval_ranks[p][cand]["Rang officiel"] = official_order[p].index(cand) + 1
        approval_ranks[p][cand]["Rang officiel national"] = official_national_order.index(cand) + 1

approval_ranks_all = {}

for cand in candidates:
    approval_ranks_all[cand] = {}
    approval_ranks_all[cand]["Score brut"] = sum(sum(v["approval"][cand] for v in vt[p]) for p in range(3))
    approval_ranks_all[cand]["Score corrigé"] = sum(sum(corrections_all[v["official"]] * v["approval"][cand] for v in vt[p] if v["official"] in candidates) for p in range(3))
    approval_ranks_all[cand]["Score corrigé national"] = sum(sum(corrections_national_all[v["official"]] * v["approval"][cand] for v in vt[p] if v["official"] in candidates) for p in range(3))
    approval_ranks_all[cand]["Moyenne du score brut"] = approval_ranks_all[cand]["Score brut"] / nb_voters_all
raw_order = sorted(candidates,
                   key=lambda cand: approval_ranks_all[cand]["Score brut"], reverse=True)
corrected_order = sorted(candidates,
                         key=lambda cand: approval_ranks_all[cand]["Score corrigé"], reverse=True)    
corrected_order_national = sorted(candidates,
                                  key=lambda cand: approval_ranks_all[cand]["Score corrigé national"], reverse=True)
for cand in candidates:
    approval_ranks_all[cand]["Rang brut"] = raw_order.index(cand) + 1
    approval_ranks_all[cand]["Rang corrigé"] = corrected_order.index(cand) + 1
    approval_ranks_all[cand]["Rang corrigé national"] = corrected_order_national.index(cand) + 1
    approval_ranks_all[cand]["Rang officiel"] = official_order_all.index(cand) + 1
    approval_ranks_all[cand]["Rang officiel national"] = official_national_order.index(cand) + 1

        
evaluation_ranks = [{}, {}, {}]

for p in range(3):
    for cand in candidates:
        evaluation_ranks[p][cand] = {}
        evaluation_ranks[p][cand]["Score brut"] = sum(v["evaluation"][cand]
                                                      if v["evaluation"][cand] is not None else 0
                                                      for v in vt[p])
        evaluation_ranks[p][cand]["Score corrigé"] = sum(corrections[p][v["official"]] * v["evaluation"][cand]
                                                         if v["evaluation"][cand] is not None else 0
                                                         for v in vt[p] if v["official"] in candidates)
        evaluation_ranks[p][cand]["Score corrigé national"] = sum(corrections_national[p][v["official"]] * v["evaluation"][cand]
                                                                  if v["evaluation"][cand] is not None else 0
                                                                  for v in vt[p] if v["official"] in candidates)
        evaluation_ranks[p][cand]["Moyenne du score brut"] = evaluation_ranks[p][cand]["Score brut"] / nb_voters[p]
    raw_order = sorted(candidates,
                       key=lambda cand: evaluation_ranks[p][cand]["Score brut"], reverse=True)
    corrected_order = sorted(candidates,
                             key=lambda cand: evaluation_ranks[p][cand]["Score corrigé"], reverse=True)        
    corrected_order_national = sorted(candidates,
                                      key=lambda cand: evaluation_ranks[p][cand]["Score corrigé national"], reverse=True)        
    for cand in candidates:
        evaluation_ranks[p][cand]["Rang brut"] = raw_order.index(cand) + 1
        evaluation_ranks[p][cand]["Rang corrigé"] = corrected_order.index(cand) + 1
        evaluation_ranks[p][cand]["Rang corrigé national"] = corrected_order_national.index(cand) + 1
        evaluation_ranks[p][cand]["Rang officiel"] = official_order[p].index(cand) + 1
        evaluation_ranks[p][cand]["Rang officiel national"] = official_national_order.index(cand) + 1

evaluation_ranks_all = {}
        
for cand in candidates:
    evaluation_ranks_all[cand] = {}
    evaluation_ranks_all[cand]["Score brut"] = sum(sum(v["evaluation"][cand]
                                                       if v["evaluation"][cand] is not None else 0
                                                       for v in vt[p]) for p in range(3))
    evaluation_ranks_all[cand]["Score corrigé"] = sum(sum(corrections_all[v["official"]] * v["evaluation"][cand]
                                                          if v["evaluation"][cand] is not None else 0
                                                          for v in vt[p]
                                                          if v["official"] in candidates) for p in range(3))
    evaluation_ranks_all[cand]["Score corrigé national"] = sum(sum(corrections_national_all[v["official"]] * v["evaluation"][cand]
                                                                   if v["evaluation"][cand] is not None else 0
                                                                   for v in vt[p]
                                                                   if v["official"] in candidates) for p in range(3))
    evaluation_ranks_all[cand]["Moyenne du score brut"] = evaluation_ranks_all[cand]["Score brut"] / nb_voters_all
raw_order = sorted(candidates,
                   key=lambda cand: evaluation_ranks_all[cand]["Score brut"], reverse=True)
corrected_order = sorted(candidates,
                         key=lambda cand: evaluation_ranks_all[cand]["Score corrigé"], reverse=True)        
corrected_order_national = sorted(candidates,
                                  key=lambda cand: evaluation_ranks_all[cand]["Score corrigé national"], reverse=True)        
for cand in candidates:
    evaluation_ranks_all[cand]["Rang brut"] = raw_order.index(cand) + 1
    evaluation_ranks_all[cand]["Rang corrigé"] = corrected_order.index(cand) + 1
    evaluation_ranks_all[cand]["Rang corrigé national"] = corrected_order_national.index(cand) + 1
    evaluation_ranks_all[cand]["Rang officiel"] = official_order_all.index(cand) + 1
    evaluation_ranks_all[cand]["Rang officiel national"] = official_national_order.index(cand) + 1

        
#### SAUVEGARDE DES RANGS ###########################

for p in range(3):
    with open("approval-scores-vt{}.csv".format(p + 1), "w") as file:
        writer = csv.writer(file, delimiter=',')
        row = ["Candidat", "Score brut", "Score corrigé", "Score corrigé national", "Moyenne du score brut",
               "Rang brut", "Rang corrigé", "Rang corrigé national", "Rang officiel", "Rang officiel national"]
        writer.writerow(row)
        for cand in cc_order:
            row = [
                cand,
                approval_ranks[p][cand]["Score brut"],
                approval_ranks[p][cand]["Score corrigé"],
                approval_ranks[p][cand]["Score corrigé national"],
                approval_ranks[p][cand]["Moyenne du score brut"],
                approval_ranks[p][cand]["Rang brut"],
                approval_ranks[p][cand]["Rang corrigé"],
                approval_ranks[p][cand]["Rang corrigé national"],
                approval_ranks[p][cand]["Rang officiel"],
                approval_ranks[p][cand]["Rang officiel national"]
            ]
            writer.writerow(row)
    with open("evaluation-scores-vt{}.csv".format(p + 1), "w") as file:
        writer = csv.writer(file, delimiter=',')
        row = ["Candidat", "Score brut", "Score corrigé", "Score corrigé national", "Moyenne du score brut",
               "Rang brut", "Rang corrigé", "Rang corrigé national", "Rang officiel", "Rang officiel national"]
        writer.writerow(row)
        for cand in cc_order:
            row = [
                cand,
                evaluation_ranks[p][cand]["Score brut"],
                evaluation_ranks[p][cand]["Score corrigé"],
                evaluation_ranks[p][cand]["Score corrigé national"],
                evaluation_ranks[p][cand]["Moyenne du score brut"],
                evaluation_ranks[p][cand]["Rang brut"],
                evaluation_ranks[p][cand]["Rang corrigé"],
                evaluation_ranks[p][cand]["Rang corrigé national"],
                evaluation_ranks[p][cand]["Rang officiel"],
                evaluation_ranks[p][cand]["Rang officiel national"]
            ]
            writer.writerow(row)

with open("approval-scores-all.csv", "w") as file:
    writer = csv.writer(file, delimiter=',')
    row = ["Candidat", "Score brut", "Score corrigé", "Score corrigé national", "Moyenne du score brut",
           "Rang brut", "Rang corrigé", "Rang corrigé national", "Rang officiel", "Rang officiel national"]
    writer.writerow(row)
    for cand in cc_order:
        row = [
            cand,
            approval_ranks_all[cand]["Score brut"],
            approval_ranks_all[cand]["Score corrigé"],
            approval_ranks_all[cand]["Score corrigé national"],
            approval_ranks_all[cand]["Moyenne du score brut"],
            approval_ranks_all[cand]["Rang brut"],
            approval_ranks_all[cand]["Rang corrigé"],
            approval_ranks_all[cand]["Rang corrigé national"],
            approval_ranks_all[cand]["Rang officiel"],
            approval_ranks_all[cand]["Rang officiel national"]
        ]
        writer.writerow(row)
with open("evaluation-scores-all.csv", "w") as file:
    writer = csv.writer(file, delimiter=',')
    row = ["Candidat", "Score brut", "Score corrigé", "Score corrigé national", "Moyenne du score brut",
           "Rang brut", "Rang corrigé", "Rang corrigé national", "Rang officiel", "Rang officiel national"]
    writer.writerow(row)
    for cand in cc_order:
        row = [
            cand,
            evaluation_ranks_all[cand]["Score brut"],
            evaluation_ranks_all[cand]["Score corrigé"],
            evaluation_ranks_all[cand]["Score corrigé national"],
            evaluation_ranks_all[cand]["Moyenne du score brut"],
            evaluation_ranks_all[cand]["Rang brut"],
            evaluation_ranks_all[cand]["Rang corrigé"],
            evaluation_ranks_all[cand]["Rang corrigé national"],
            evaluation_ranks_all[cand]["Rang officiel"],
            evaluation_ranks_all[cand]["Rang officiel national"]
        ]
        writer.writerow(row)


#### CALCUL DE LA CO-APPROBATION ###########################

co_approval_raw = [{}, {}, {}]
co_approval_corrected = [{}, {}, {}]
co_approval_corrected_national = [{}, {}, {}]
official_approval_raw = [{}, {}, {}]
official_approval_corrected = [{}, {}, {}]
official_approval_corrected_national = [{}, {}, {}]

for p in range(3):
    co_approval_raw[p] = {}
    co_approval_corrected[p] = {}
    co_approval_corrected_national[p] = {}
    official_approval_raw[p] = {}
    official_approval_corrected[p] = {}
    official_approval_corrected_national[p] = {}
    for c1 in candidates:
        co_approval_raw[p][c1] = {}
        co_approval_corrected[p][c1] = {}
        co_approval_corrected_national[p][c1] = {}
        official_approval_raw[p][c1] = {}
        official_approval_corrected[p][c1] = {}
        official_approval_corrected_national[p][c1] = {}
        for c2 in candidates:
            co_approval_raw[p][c1][c2] = 0
            co_approval_corrected[p][c1][c2] = 0
            co_approval_corrected_national[p][c1][c2] = 0
            official_approval_raw[p][c1][c2] = 0
            official_approval_corrected[p][c1][c2] = 0
            official_approval_corrected_national[p][c1][c2] = 0

    for v in vt[p]:
        approval_set = [c for c in candidates if v["approval"][c] == 1]
        for c1, c2 in itertools.product(approval_set, approval_set):
            co_approval_raw[p][c1][c2] += 1
            co_approval_corrected[p][c1][c2] += (
                corrections[p][v["official"]] if v["official"] in candidates else 0)
            co_approval_corrected_national[p][c1][c2] += (
                corrections_national[p][v["official"]] if v["official"] in candidates else 0)
        for c2 in approval_set:
            if v["official"] in candidates:
                official_approval_raw[p][v["official"]][c2] += 1
                official_approval_corrected[p][v["official"]][c2] += corrections[p][v["official"]]
                official_approval_corrected[p][v["official"]][c2] += corrections_national[p][v["official"]]


co_approval_raw_all = {}
co_approval_corrected_all = {}
co_approval_corrected_national_all = {}
official_approval_raw_all = {}
official_approval_corrected_all = {}
official_approval_corrected_national_all = {}

co_approval_raw_all = {}
co_approval_corrected_all = {}
co_approval_corrected_national_all = {}
official_approval_raw_all = {}
official_approval_corrected_all = {}
official_approval_corrected_national_all = {}
for c1 in candidates:
    co_approval_raw_all[c1] = {}
    co_approval_corrected_all[c1] = {}
    co_approval_corrected_national_all[c1] = {}
    official_approval_raw_all[c1] = {}
    official_approval_corrected_all[c1] = {}
    official_approval_corrected_national_all[c1] = {}
    for c2 in candidates:
        co_approval_raw_all[c1][c2] = 0
        co_approval_corrected_all[c1][c2] = 0
        co_approval_corrected_national_all[c1][c2] = 0
        official_approval_raw_all[c1][c2] = 0
        official_approval_corrected_all[c1][c2] = 0
        official_approval_corrected_national_all[c1][c2] = 0

for p in range(3):
    for v in vt[p]:
        approval_set = [c for c in candidates if v["approval"][c] == 1]
        for c1, c2 in itertools.product(approval_set, approval_set):
            co_approval_raw_all[c1][c2] += 1
            co_approval_corrected_all[c1][c2] += (
                corrections[p][v["official"]] if v["official"] in candidates else 0)
            co_approval_corrected_national_all[c1][c2] += (
                corrections_national[p][v["official"]] if v["official"] in candidates else 0)
        for c2 in approval_set:
            if v["official"] in candidates:
                official_approval_raw_all[v["official"]][c2] += 1
                official_approval_corrected_all[v["official"]][c2] += corrections[p][v["official"]]
                official_approval_corrected_all[v["official"]][c2] += corrections_national[p][v["official"]]

#### SAUVEGARDE DE LA COAPPROBATION ###########################

for p in range(3):
    with open("coapproval-vt{}-raw.csv".format(p + 1), "w") as file:
        writer = csv.writer(file, delimiter=',')
        row = ["Candidat approuvé"] + cc_order
        writer.writerow(row)
        for c1 in cc_order:
            row = [c1] + [co_approval_raw[p][c1][c2] for c2 in cc_order]
            writer.writerow(row)
    with open("coapproval-vt{}-corrected.csv".format(p + 1), "w") as file:
        writer = csv.writer(file, delimiter=',')
        row = ["Candidat approuvé"] + cc_order
        writer.writerow(row)
        for c1 in cc_order:
            row = [c1] + [co_approval_corrected[p][c1][c2] for c2 in cc_order]
            writer.writerow(row)
    with open("coapproval-vt{}-corrected-national.csv".format(p + 1), "w") as file:
        writer = csv.writer(file, delimiter=',')
        row = ["Candidat approuvé"] + cc_order
        writer.writerow(row)
        for c1 in cc_order:
            row = [c1] + [co_approval_corrected_national[p][c1][c2] for c2 in cc_order]
            writer.writerow(row)
    with open("official-approval-vt{}-raw.csv".format(p + 1), "w") as file:
        writer = csv.writer(file, delimiter=',')
        row = ["Candidat approuvé"] + cc_order
        writer.writerow(row)
        for c2 in cc_order:
            row = [c2] + [co_approval_raw[p][c1][c2] for c1 in cc_order]
            writer.writerow(row)
    with open("official-approval-vt{}-corrected.csv".format(p + 1), "w") as file:
        writer = csv.writer(file, delimiter=',')
        row = ["Candidat approuvé"] + cc_order
        writer.writerow(row)
        for c2 in cc_order:
            row = [c2] + [co_approval_corrected[p][c1][c2] for c1 in cc_order]
            writer.writerow(row)
    with open("official-approval-vt{}-corrected-national.csv".format(p + 1), "w") as file:
        writer = csv.writer(file, delimiter=',')
        row = ["Candidat approuvé"] + cc_order
        writer.writerow(row)
        for c2 in cc_order:
            row = [c2] + [co_approval_corrected_national[p][c1][c2] for c1 in cc_order]
            writer.writerow(row)

with open("coapproval-all-raw.csv", "w") as file:
    writer = csv.writer(file, delimiter=',')
    row = ["Candidat approuvé"] + cc_order
    writer.writerow(row)
    for c1 in cc_order:
        row = [c1] + [co_approval_raw_all[c1][c2] for c2 in cc_order]
        writer.writerow(row)
with open("coapproval-all-corrected.csv", "w") as file:
    writer = csv.writer(file, delimiter=',')
    row = ["Candidat approuvé"] + cc_order
    writer.writerow(row)
    for c1 in cc_order:
        row = [c1] + [co_approval_corrected_all[c1][c2] for c2 in cc_order]
        writer.writerow(row)
with open("coapproval-all-corrected-national.csv", "w") as file:
    writer = csv.writer(file, delimiter=',')
    row = ["Candidat approuvé"] + cc_order
    writer.writerow(row)
    for c1 in cc_order:
        row = [c1] + [co_approval_corrected_national_all[c1][c2] for c2 in cc_order]
        writer.writerow(row)
with open("official-approval-all-raw.csv", "w") as file:
    writer = csv.writer(file, delimiter=',')
    row = ["Candidat approuvé"] + cc_order
    writer.writerow(row)
    for c2 in cc_order:
        row = [c2] + [co_approval_raw_all[c1][c2] for c1 in cc_order]
        writer.writerow(row)
with open("official-approval-all-corrected.csv", "w") as file:
    writer = csv.writer(file, delimiter=',')
    row = ["Candidat approuvé"] + cc_order
    writer.writerow(row)
    for c2 in cc_order:
        row = [c2] + [co_approval_corrected_all[c1][c2] for c1 in cc_order]
        writer.writerow(row)
with open("official-approval-all-corrected-national.csv", "w") as file:
    writer = csv.writer(file, delimiter=',')
    row = ["Candidat approuvé"] + cc_order
    writer.writerow(row)
    for c2 in cc_order:
        row = [c2] + [co_approval_corrected_national_all[c1][c2] for c1 in cc_order]
        writer.writerow(row)

            
#### CALCUL DU DÉTAIL DES VALEURS ###########################

detail_raw = [{}, {}, {}]
detail_corrected = [{}, {}, {}]
detail_corrected_national = [{}, {}, {}]

for p in range(3):
    detail_raw[p] = {}
    detail_corrected[p] = {}
    detail_corrected_national[p] = {}
    for c1 in candidates:
        detail_raw[p][c1] = {}
        detail_corrected[p][c1] = {}
        detail_corrected_national[p][c1] = {}
        for k in range(1, nb_candidates + 1):
            detail_raw[p][c1][k] = 0
            detail_corrected[p][c1][k] = 0
            detail_corrected_national[p][c1][k] = 0

    for v in vt[p]:
        approval_set = [c for c in candidates if v["approval"][c] == 1]
        nb_approval = len(approval_set)
        for c1 in approval_set:
            detail_raw[p][c1][nb_approval] += 1
            detail_corrected[p][c1][nb_approval] += (
                corrections[p][v["official"]] if v["official"] in candidates else 0)
            detail_corrected_national[p][c1][nb_approval] += (
                corrections_national[p][v["official"]] if v["official"] in candidates else 0)

detail_raw_all = {}
detail_corrected_all = {}
detail_corrected_national_all = {}
for c1 in candidates:
    detail_raw_all[c1] = {}
    detail_corrected_all[c1] = {}
    detail_corrected_national_all[c1] = {}
    for k in range(1, nb_candidates + 1):
        detail_raw_all[c1][k] = 0
        detail_corrected_all[c1][k] = 0
        detail_corrected_national_all[c1][k] = 0

for p in range(3):
    for v in vt[p]:
        approval_set = [c for c in candidates if v["approval"][c] == 1]
        nb_approval = len(approval_set)
        for c1 in approval_set:
            detail_raw_all[c1][nb_approval] += 1
            detail_corrected_all[c1][nb_approval] += (
                corrections[p][v["official"]] if v["official"] in candidates else 0)
            detail_corrected_national_all[c1][nb_approval] += (
                corrections_national[p][v["official"]] if v["official"] in candidates else 0)

            
#### SAUVEGARDE DU DÉTAIL DES VALEURS ###########################

for p in range(3):
    with open("approval-detail-vt{}-raw.csv".format(p + 1), "w") as file:
        writer = csv.writer(file, delimiter=',')
        row = ["Candidat approuvé"] + list(range(1, nb_candidates + 1))
        writer.writerow(row)
        for c1 in cc_order:
            row = [c1] + [detail_raw[p][c1][k] for k in range(1, nb_candidates + 1)]
            writer.writerow(row)
    with open("approval-detail-vt{}-corrected.csv".format(p + 1), "w") as file:
        writer = csv.writer(file, delimiter=',')
        row = ["Candidat approuvé"] + list(range(1, nb_candidates + 1))
        writer.writerow(row)
        for c1 in cc_order:
            row = [c1] + [detail_corrected[p][c1][k] for k in range(1, nb_candidates + 1)]
            writer.writerow(row)
    with open("approval-detail-vt{}-corrected-national.csv".format(p + 1), "w") as file:
        writer = csv.writer(file, delimiter=',')
        row = ["Candidat approuvé"] + list(range(1, nb_candidates + 1))
        writer.writerow(row)
        for c1 in cc_order:
            row = [c1] + [detail_corrected_national[p][c1][k] for k in range(1, nb_candidates + 1)]
            writer.writerow(row)

with open("approval-detail-all-raw.csv", "w") as file:
    writer = csv.writer(file, delimiter=',')
    row = ["Candidat approuvé"] + list(range(1, nb_candidates + 1))
    writer.writerow(row)
    for c1 in cc_order:
        row = [c1] + [detail_raw_all[c1][k] for k in range(1, nb_candidates + 1)]
        writer.writerow(row)
with open("approval-detail-all-corrected.csv", "w") as file:
    writer = csv.writer(file, delimiter=',')
    row = ["Candidat approuvé"] + list(range(1, nb_candidates + 1))
    writer.writerow(row)
    for c1 in cc_order:
        row = [c1] + [detail_corrected_all[c1][k] for k in range(1, nb_candidates + 1)]
        writer.writerow(row)
with open("approval-detail-all-corrected-national.csv", "w") as file:
    writer = csv.writer(file, delimiter=',')
    row = ["Candidat approuvé"] + list(range(1, nb_candidates + 1))
    writer.writerow(row)
    for c1 in cc_order:
        row = [c1] + [detail_corrected_national_all[c1][k] for k in range(1, nb_candidates + 1)]
        writer.writerow(row)


#### SAUVEGARDE DU DÉTAIL DES VALEURS (ÉVALUATION) ###########################

values = []
for p in range(3):
    for v in vt[p]:
        for k in candidates:
            if v["evaluation"][k]:
                values.append(v["evaluation"][k])
values.sort()

with open("evaluation-values-all.csv", "w") as file:
    file.write("Valeur\n")
    file.write('\n'.join(str(f) for f in values))
