#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from collections import defaultdict
from random import random

from data import candidates, colors, results, coefs

orders = defaultdict(float)

for bureau in ['vt1', 'vt2', 'vt3']:
	data = '../' + bureau + '.tsv'
	data = open(data)

	columns = next(data)
	columns = columns.split()

	indices = {
		c: columns.index('EV_'+c)
		for c in candidates
	}
	VOTE = columns.index('VOTE')

	for line in data:
		words = line.split()
		vote = words[VOTE]
		try:
			coef = coefs[bureau][vote]
		except:
			assert vote == "NSPP"
			continue
		order = []
		for c in candidates:
			try:
				note = float(words[indices[c]])
			except:
				note = 0.
			order += [(-note, c)]
		order.sort()
		order = [c for _, c in order]
		orders[tuple(order)] += 1./coef


#p = [(orders[o], o) for o in orders]
#p.sort()
#for k in p:
#	print(k)


# stv et al #################################################################

#"""
def stv(orders):
	scores = defaultdict(float)
	for order in orders:
		score = orders[order]
		candidate = order[0]
		scores[candidate] += score
	return scores

def coombs(orders):
	scores = defaultdict(float)
	for order in orders:
		score = orders[order]
		candidate = order[-1]
		scores[candidate] -= score
	return scores

def iborda(orders):
	scores = defaultdict(float)
	for order in orders:
		N = len(order)
		break
	for order in orders:
		assert len(order) == N
		score = orders[order]
		for i, candidate in enumerate(order):
			scores[candidate] += (N-i)*score
	return scores
		

def transfer(candidate, orders):
	result = defaultdict(float)
	for order in orders:
		score = orders[order]
		order = list(order)
		try:
			order.remove(candidate)
		except ValueError:
			pass
		result[tuple(order)] += score
	return result

round = stv
round = coombs
round = iborda

for name, round in zip(
	['stv', 'coombs', 'borda iteratif'],
	[stv,   coombs,   iborda]):
	print(name)
	orders_copy = dict(orders)
	for r in range(len(candidates)):
		print("tour %2i:" % (r+1,), end=' ')
		scores = round(orders)
		stv_scores = stv(orders)
		for candidate in sorted(scores, key=lambda k: -scores[k]):
#			print("%-3s\t(%6.2f)" % (candidate, score))
			print("%3s (%6.2f, %6.2f)" % (candidate, scores[candidate], stv_scores[candidate]), end='\t')
		print()
#		print("tranfert de :", candidate)
		orders = transfer(candidate, orders)
	orders = orders_copy
#"""


# visualisation #############################################################

# preference tree ###########################################################

root = {}
def insert(pref, count, rank=1, tree=root):
	try:
		name = pref[rank-1]
	except:
		return
	try:
		weight, branch = tree[name]
	except KeyError:
		weight, branch = 0., {}
	weight += count
	insert(pref, count, rank+1, branch)
	tree[name] = (weight, branch)

for o in orders:
	insert(o, orders[o])


# preferences with cumulative weights #######################################

def tree2lists(prefix=None, tree=root):
	if prefix == None:
		prefix = [], []
	if tree == {}:
		return [prefix]

	names, weights = prefix
	lists = []
	for name in tree:
		weight, branch = tree[name]
		lists += tree2lists((names + [name], weights + [weight]), branch)
	return lists

preference_lists = tree2lists()
N = len(preference_lists)

weight = sum(weights[-1] for _, weights in preference_lists)
widths = list(weights[-1]/weight for _, weights in preference_lists)


# permutations ##############################################################

def candidate_key(candidate):
	def key(i):
		names, weights = preference_lists[i]
		try:
			rank = 11 - names.index(candidate)
		except ValueError:
			rank = 0
		return rank, -weights[0]
#	import random
#	def key(i):
#		return random.random()
	return key


# visualisation #############################################################

from OpenGL.GLUT import *
from OpenGL.GL import *


def permutation(c=None):
	return sorted(range(N), key=candidate_key(c), reverse=True)

def layout(order):
	u = 0
	us = [None for _ in preference_lists]
	for j in order:
		us[j] = u
		u += widths[j]
	return us

order = permutation("JLM")
positions = layout(order)

def interpolate(start_time):
	global positions
	glutPostRedisplay()
	if positions == target_positions:
		return
	for i, (tp, p) in enumerate(zip(target_positions, positions)):
		dp = tp - p
		if dp > 0:
			p = min(p+20/WIDTH, tp)
		else:
			p = max(p-20/WIDTH, tp)
		positions[i] = p
	glutTimerFunc(0, interpolate, start_time)


# glut ######################################################################

def reshape(width, height):
	"""window reshape callback."""
	global WIDTH, HEIGHT
	WIDTH, HEIGHT = width, height
	glViewport(0, 0, width, height)
	
	glMatrixMode(GL_PROJECTION)
	glLoadIdentity()
	glOrtho(0, width, height, 0, -1, 1)
	
	glMatrixMode(GL_MODELVIEW)
	glLoadIdentity()

def rgb(h):
	h = '%06x' % h
	return tuple(int(u, 16)/256 for u in (h[:2], h[2:4], h[4:]))

def display():
	"""window redisplay callback."""
	glClear(GL_COLOR_BUFFER_BIT)
	h = HEIGHT/len(candidates)
	glBegin(GL_QUADS)
	for (names, _), u, du in zip(preference_lists, positions, widths):
		x0 = u*WIDTH
		x1 = x0 + du*WIDTH
		for i, name in enumerate(names):
			glColor(*rgb(colors[name]))
			glVertex(x0,  i*h)
			glVertex(x0, (i+1)*h)
			glVertex(x1, (i+1)*h)
			glVertex(x1,  i*h)
	glEnd()
	glutSwapBuffers()


def keyboard(c, x=0, y=0):
	"""keyboard callback."""
	if c == b'q':
		sys.exit(0)
	elif c in b'0123456789r':
		global order, target_positions
		try:
			order = permutation(candidates[int(c)-1])
		except:
			order = permutation()
		target_positions = layout(order)
		glutTimerFunc(0, interpolate, glutGet(GLUT_ELAPSED_TIME))
	glutPostRedisplay()


WIDTH, HEIGHT = 800, 600

glutInit(sys.argv)
glutInitWindowSize(WIDTH, HEIGHT)
glutInitDisplayString('rgba double') # hidpi')
glutCreateWindow(sys.argv[0])

glutReshapeFunc(reshape)
glutDisplayFunc(display)
glutKeyboardFunc(keyboard)
#glutMouseFunc(mouse)
#glutMotionFunc(motion)

glutMainLoop()
