#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from seagull import scenegraph as sg
from seagull.xml import serialize
	

from data import candidates, colors, results, coefs

lines = {
	c: []
	for c in candidates + ['B']
}

LEFT_RIGHT = ["NA", "PP", "JLM", "BH", "EM", "JL", "FF", "NDA", "MLP", "JC", "FA"]

for bureau in ['vt1', 'vt2', 'vt3']:
	data = '../' + bureau + '.tsv'
	data = open(data)

	columns = next(data)
	columns = columns.split()

	indices = {
		c: columns.index('EV_'+c)
		for c in candidates
	}
	VOTE = columns.index('VOTE')

	def note(words, c):
		try:
			note = float(words[indices[c]])
		except:
			note = 0.
		if note < 0.: note = 0.
		if note > 1.: note = 1.
		return note

	for line in data:
		words = line.split()
		vote = words[VOTE]
		try:
			coef = coefs[bureau][vote]
		except:
			assert vote == "NSPP"
			continue
		
		points = [
			(i, 5*note(words, c))
			for i, c in enumerate(LEFT_RIGHT)
		]
		lines[vote].append(sg.Polyline(points=points))

scene = sg.Group(
	transform=[sg.Scale(100, 100), sg.Scale(1, -1), sg.Translate(0, -5)],
	stroke_width=.01,
	children=[
		sg.Group(
			fill=None,
			stroke="#%06x" % colors[c],
#			stroke=sg.Color.black,
			stroke_opacity=.25,
			children=lines[c]
		)
		for c in LEFT_RIGHT
	],
)

print(serialize(scene, bbox=((0, 0), ((len(candidates)-1)*100, 500))))
