#!/usr/bin/python3

import sys
import numpy as np
from sklearn.decomposition import PCA
from mpl_toolkits.mplot3d import Axes3D
import matplotlib.pyplot as plt

from data import candidates, colors, results, coefs

REDRESSE = False

def pca_1d():
    pca = PCA(n_components=1)
    pca.fit(X)
    X_pca = pca.transform(X)

    fig = plt.figure()

    colors['B'] = 0xffffff
    colors['NSPP'] = 0xcccccc

    for candidate in colors:
        nb_voters = sum(1 for x in range(len(X_pca)) if true_vote[x] == candidate)
        voters_x = [X_pca[x, 0]
                    for x in range(len(X_pca)) if true_vote[x] == candidate]
        voters_y = [0
                    for x in range(len(X_pca)) if true_vote[x] == candidate]
        voters_x = np.array(voters_x)
        voters_y = np.array(voters_y)
        plt.scatter(voters_x, voters_y,
                    color="#%06x" % (colors[candidate]),
                    marker='+',
                    alpha=0.5)
        if (nb_voters):
            centroid = sum(voters_x) / nb_voters, sum(voters_y) / nb_voters
            plt.scatter(centroid[0], centroid[1],
                        color="#%06x" % (colors[candidate]),
                        marker='*',
                        s=100,
                        alpha=1)
            plt.text(centroid[0], centroid[1] + 0.002, candidate,
                     color="#%06x" % (colors[candidate]))

    print(candidates)
    print(pca.components_)
    print("Version classée :")
    print([candidates[c] for c in sorted(range(len(candidates)), key=lambda c: pca.components_[0,c])])
    plt.show()

def pca_2d():
    pca = PCA(n_components=2)
    pca.fit(X)
    X_pca = pca.transform(X)

    fig = plt.figure()

    colors['B'] = 0xffffff
    colors['NSPP'] = 0xcccccc

    for candidate in colors:
        nb_voters = sum(1 for x in range(len(X_pca)) if true_vote[x] == candidate)
        voters_x = [X_pca[x, 0]
                    for x in range(len(X_pca)) if true_vote[x] == candidate]
        voters_y = [X_pca[x, 1]
                    for x in range(len(X_pca)) if true_vote[x] == candidate]
        voters_x = np.array(voters_x)
        voters_y = np.array(voters_y)
        plt.scatter(voters_x, voters_y,
                    color="#%06x" % (colors[candidate]),
                    marker='+',
                    alpha=0.5)
        if (nb_voters):
            centroid = sum(voters_x) / nb_voters, sum(voters_y) / nb_voters
            plt.scatter(centroid[0], centroid[1],
                        color="#%06x" % (colors[candidate]),
                        marker='*',
                        s=100,
                        alpha=1)
            plt.text(centroid[0] + 0.05, centroid[1], candidate,
                     color="#%06x" % (colors[candidate]))

    print(candidates)
    print(pca.components_)
    print("Version classée :")
    print("Axe X : " + str([candidates[c] for c in sorted(range(len(candidates)), key=lambda c: pca.components_[0,c])]))
    print("Axe Y : " + str([candidates[c] for c in sorted(range(len(candidates)), key=lambda c: pca.components_[1,c])]))
    plt.show()


def pca_3d():
    pca = PCA(n_components=3)
    pca.fit(X)
    X_pca = pca.transform(X)

    fig = plt.figure()
    ax = fig.add_subplot(111, projection='3d')

    colors['B'] = 0xffffff
    colors['NSPP'] = 0xcccccc

    for candidate in colors:
        nb_voters = sum(1 for x in range(len(X_pca)) if true_vote[x] == candidate)
        voters_x = [X_pca[x, 0]
                    for x in range(len(X_pca)) if true_vote[x] == candidate]
        voters_y = [X_pca[x, 1]
                    for x in range(len(X_pca)) if true_vote[x] == candidate]
        voters_z = [X_pca[x, 2]
                    for x in range(len(X_pca)) if true_vote[x] == candidate]
        voters_x = np.array(voters_x)
        voters_y = np.array(voters_y)
        voters_z = np.array(voters_z)
        ax.scatter(voters_x, voters_y, voters_z,
                   color="#%06x" % (colors[candidate]),
                   marker='+',
                   alpha=0.5)
        if (nb_voters):
            centroid = sum(voters_x) / nb_voters, sum(voters_y) / nb_voters,\
                       sum(voters_z) / nb_voters
            ax.scatter(centroid[0], centroid[1], centroid[2],
                       color="#%06x" % (colors[candidate]),
                       marker='*',
                       s=100,
                       alpha=1)
            ax.text(centroid[0] + 0.05, centroid[1], centroid[2], candidate,
                    color="#%06x" % (colors[candidate]))

    plt.show()


voting_profile = []
true_vote = []

for bureau in ['vt1', 'vt2', 'vt3']:
    data = '../' + bureau + '.tsv'
    data = open(data)

    columns = next(data)
    columns = columns.split()

    indices = {
        c: columns.index('EV_'+c)
        for c in candidates
    }
    VOTE = columns.index('VOTE')

    for line in data:
        words = line.split()
        vote = words[VOTE]
        try:
            coef = coefs[bureau][vote]
        except KeyError:
            assert vote == "NSPP"
            continue
        vote_vector = []
        for c in candidates:
            try:
                note = float(words[indices[c]])
            except ValueError:
                note = 0.
            vote_vector.append(note)
        if REDRESSE:
            for k in range(int(10 * coef)):
                voting_profile += [vote_vector]
                true_vote += [vote]
        else:
            voting_profile += [vote_vector]
            true_vote += [vote]

X = np.array(voting_profile)

if len(sys.argv) > 1:
    if sys.argv[1] == '1':
        pca_1d()
    elif sys.argv[1] == '2':
        pca_2d()
    else:
        pca_3d()
else:
    sys.stderr.write("Un argument attendu: 1, 2, ou 3.\n")
