#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""template.py

template for processing VT data
"""

from math import log
from data import candidates, left_right, results

voters = []
weights = {
        c: 1.
        for c in candidates + ["B"]
}
resultats = {
        c: results["france"][c][0]
        for c in candidates
}
resultats['B'] = results["france"]["Blanc"]


for bureau in ['vt1', 'vt2', 'vt3']:
        data = '../' + bureau + '.tsv'
        data = open(data)

        columns = next(data)
        columns = columns.split()

        indices = {
                c: columns.index('EV_'+c)
                for c in candidates
        }
        VOTE = columns.index('VOTE')

        for line in data:
                words = line.split()
                vote = words[VOTE]
                if vote == "NSPP":
                        continue
                notes = []
                for c in candidates:
                        try:
                                note = float(words[indices[c]])
                        except:
                                note = 0.
                        if note < 0.: note = 0.
                        if note > 1.: note = 1.
                        notes += [note]
                if sum(notes) == 0.:
                        continue
                voters += [(vote, 1., notes)]

N = len(voters)
print("considering", N, "voters")

counts = {
        c: sum(vote == c for vote, _, _ in voters)/N
        for c in candidates + ['B']
}

print(counts)

alpha, beta = .25, 2

#while True:
for _ in range(10):
        # computing election result with official vote
        for c in candidates + ["B"]:
                score = 0.
                total = 0.
                for vote, w, _ in voters:
                        if vote != "B" or True:
                                total += w
                        if vote == c:
                                score += w
                wn = (score/total*100) / resultats[c]
                print("%s: %.02f (%.02f vs. %.02f)" % (c, wn, score/total*100, resultats[c]))
                weights[c] = wn
        
        voters_n = []
        ws = []
        for vote, w_o, notes in voters:
#                w_n = sum(1./weights[c] * note for note, c in zip(notes, candidates))
#                w_n = sum(1./(weights[c]**beta) * note for note, c in zip(notes, candidates))
                w_n = sum(counts[c]/(weights[c]**beta) * note for note, c in zip(notes, candidates))
                w_n /= sum(notes)
                w_n = alpha*(w_n - w_o) + w_o # stabilization
                ws += [(w_n, w_o, vote)]
                voters_n += [(vote, w_n, notes)]
        voters = voters_n
        ws.sort()
#        print(ws[0], ws[len(ws)//2], ws[-1], ws[-1][0]/ws[0][0])
#        if input("continue [Y/n]?") == 'n':
#                break

if False:
        # show actual weights
        voters.sort(key=lambda u: u[1]/weights[u[0]])
        for vote, w, notes in voters:
                print(w/weights[vote], vote, *notes)


evals = {
        c: 0.
        for c in candidates        
}

votes = {
        c: 0.
        for c in candidates        + ['B']
}

total = 0.
for vote, w, notes in voters:
        w /= weights[vote]
        if vote != "B" or True:
                total += w
        votes[vote] += w
        for note, c in zip(notes, candidates):
                evals[c] += note*w

for c in candidates:
        print("%s: %0.2f (%.02f vs. %.02f)" % (c, evals[c]/total*100, votes[c]/total*100, resultats[c]))
print("'B': (%.02f vs. %.02f)" % (votes['B']/total*100, resultats['B']))


if False:
        # visualisation

        VOTES = left_right + ['B']
        output = open('ponderations.txt', 'w')
        print("#", *VOTES, file=output)

        sums = {
                v: 0.
                for v in VOTES
        }

        print(-float('inf'), *(sums[v] for v in VOTES), file=output)
        voters.sort(key=lambda voter: voter[1]/weights[voter[0]])
        
        vote, w, _ = voters[0]
        last_weight = log(w/weights[vote], 2)-.5
        for vote, w, _ in voters:
                weight = log(w/weights[vote], 2)
                if last_weight != weight:
                        print(last_weight, *(sums[v] for v in VOTES), file=output)
                        last_weight = weight
                sums[vote] += 1
        
        print(weight,       *(sums[v] for v in VOTES), file=output)
        print(weight+.5,    *(sums[v] for v in VOTES), file=output)
        print(float('inf'), *(sums[v] for v in VOTES), file=output)
        output.close()
