#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from seagull import scenegraph as sg
from seagull.xml import serialize
	

from data import candidates, colors, results, coefs, left_right

dots = {
	c: []
	for c in candidates + ['B']
}

circle = sg.Circle(r=.01)

for bureau in ['vt1', 'vt2', 'vt3']:
	data = '../' + bureau + '.tsv'
	data = open(data)

	columns = next(data)
	columns = columns.split()

	indices = {
		c: columns.index('EV_'+c)
		for c in candidates
	}
	VOTE = columns.index('VOTE')

	def note(words, c):
		try:
			note = float(words[indices[c]])
		except:
			note = 0.
		if note < 0.: note = 0.
		if note > 1.: note = 1.
		return note

	for line in data:
		words = line.split()
		vote = words[VOTE]
		try:
			coef = coefs[bureau][vote]
		except:
			assert vote == "NSPP"
			continue
		
		for i, ci in enumerate(left_right):
			ni = note(words, ci)
			for j, cj in enumerate(left_right):
				if j > i: break
				nj = note(words, cj)
				dots[vote].append(sg.Use(circle, transform=[sg.Translate(i*1.1+ni, j*1.1+nj)]))

scene = sg.Group(
	transform=[sg.Scale(100), sg.Scale(1, -1), sg.Translate(0, -len(candidates)*1.1)],
	stroke=None,
	children=[
		sg.Group(
			fill="#%06x" % colors[c],
#			fill=sg.Color.black,
			fill_opacity=.1,
			children=dots[c]
		)
		for c in left_right
	],
)

print(serialize(scene, bbox=((0, 0), (len(candidates)*110, len(candidates)*110))))
